#HW3utils
import numpy as np
import tensorflow as tf
import matplotlib.pyplot as plt

def masked_loss_function(Y,Yhat,mask):
    cce = tf.keras.losses.categorical_crossentropy
    per_sample_losses = cce(Y,Yhat)
    sample_weights = tf.cast(mask,"float64")
    correction_factor = tf.reduce_mean(sample_weights)
    masked_losses = per_sample_losses * sample_weights
    loss = tf.reduce_mean(masked_losses)/correction_factor
    return loss

def masked_accuracy_function(Y,Yhat,mask):
    preds = np.argmax(Yhat,axis=1)
    targets = np.argmax(Y,axis=1)
    accuracy = (preds==targets)
    sample_weights = tf.cast(mask,"float64")
    correction_factor = tf.reduce_mean(sample_weights)
    masked_accuracy = accuracy*sample_weights
    accuracy = tf.reduce_mean(masked_accuracy)/correction_factor
    return accuracy

def training_curves(history):
    train_loss = history[:,0]
    test_loss = history[:,1]
    train_acc = history[:,2]
    test_acc = history[:,3]
    nsample,_ = history.shape
    epochs = np.arange(1,nsample+1)


    figure, axis = plt.subplots(1,2,figsize=(10,4))
    axis[0].plot(epochs, train_loss, "b--", label="training loss")
    axis[0].plot(epochs, test_loss, "b", label="testing loss")
    axis[0].set_title("Training and Testing Losses")
    axis[0].set_xlabel("Epochs")
    axis[0].set_ylabel("Loss")
    axis[0].legend()


    axis[1].plot(epochs, train_acc, "b--",label="Training acc")
    axis[1].plot(epochs, test_acc, "b", label="Testing acc")
    axis[1].set_title("Training and Testing accuracy")
    axis[1].set_xlabel("Epochs")
    axis[1].set_ylabel("Accuracy")
    axis[1].legend()
    
def loss_function(Y, Yhat):
    cce = tf.keras.losses.categorical_crossentropy
    per_sample_losses = cce(Y,Yhat)
    loss = tf.reduce_mean(per_sample_losses)
    return loss

def accuracy_function(Y,Yhat):
    preds = np.argmax(Yhat, axis=1)
    targets = np.argmax(Y,axis=1)
    accuracy = tf.cast((preds==targets),"float64")
    accuracy = tf.reduce_mean(accuracy)
    return accuracy

